/*
 * Copyright 2024 Arne Vajh�j.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.vms.tig;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

import dk.vajhoej.vms.tig.client.ClientGen;
import dk.vajhoej.vms.tig.server.ServerGen;

public class Gen {
	private static void genServer(ServerGen g, String dirnam, boolean mt, String ns, String imgnam, int port, Config.MethodList methods) {
		try {
			g.open(dirnam + System.getProperty("file.separator") + g.fileName(imgnam));
			g.genClassHeader(ns, g.className(imgnam), mt, port);
			g.genProcessorStart();
			for(Method m : methods.getMethods()) {
				g.genProcessorFragment(imgnam, m.getName(), m.getArgs() != null ? m.getArgs() : new Method.ArgList());
			}
			g.genProcessorFinish();
			g.genClassFooter(g.className(imgnam));
			g.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	private static void genClientMethod(ClientGen g, Method m) {
		g.genMethodHeader(m.getName(), m.getArgs() != null ? m.getArgs() : new Method.ArgList());
		g.genMethodBody(m.getArgs() != null ? m.getArgs() : new Method.ArgList());
		g.genMethodFooter(m.getArgs() != null ? m.getArgs() : new Method.ArgList());
	}
	private static void genClient(ClientGen g, String dirnam, String ns, String imgnam, int port, Config.MethodList methods) {
		try {
			g.open(dirnam + System.getProperty("file.separator") + g.fileName(imgnam));
			g.genClassHeader(ns, g.className(imgnam));
			g.genCtor(g.className(imgnam), port);
			for(Method m : methods.getMethods()) {
				genClientMethod(g, m);
			}
			g.genClassFooter();
			g.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	private static void process(String cfgfnm, Config cfg, String srvdir, String clidir) throws InstantiationException, IllegalAccessException, ClassNotFoundException {
		for(String lang : cfg.getServer().getLangs()) {
			System.out.printf("Processing %s (%s) with %s\n", cfgfnm, cfg.getImage(), lang);
			ServerGen g = (ServerGen)Class.forName(lang).newInstance();
			genServer(g, srvdir, cfg.isMt(), cfg.getNamespace(), cfg.getImage(), cfg.getPort(), cfg.getMethods());
		}
		for(String lang : cfg.getClient().getLangs()) {
			System.out.printf("Processing %s (%s) with %s\n", cfgfnm, cfg.getImage(), lang);
			ClientGen g = (ClientGen)Class.forName(lang).newInstance();
			genClient(g, clidir, cfg.getNamespace(), cfg.getImage(), cfg.getPort(), cfg.getMethods());
		}
	}
	public static void main(String[] args) throws JAXBException, IOException, InstantiationException, IllegalAccessException, ClassNotFoundException {
		if(args.length == 3) {
			String cfgfnm = args[0];
			String srvdir = args[1];
			String clidir = args[2];
	        JAXBContext jxbctx = JAXBContext.newInstance(Config.class);
	        Unmarshaller um = jxbctx.createUnmarshaller();
	        InputStream is = new FileInputStream(cfgfnm);
	        Config cfg = (Config)um.unmarshal(is);
	        is.close();
	        process(cfgfnm, cfg, srvdir, clidir);
		} else {
			System.out.printf("Usage: java -cp tig.jar %s <config file> <server dir> <client dir>\n", Gen.class.getName());
		}
	}
}
